/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.conqat.lib.commons.collections.CollectionUtils;
import org.conqat.lib.commons.collections.UnmodifiableList;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.extension.IExecutionTarget;
import org.fortiss.tooling.kernel.extension.data.ExecutionConfigTransformationContext;
import org.fortiss.tooling.kernel.extension.data.TransformationProviderChain;
import org.fortiss.tooling.kernel.extension.exception.ChainTransformationFailedException;
import org.fortiss.tooling.kernel.model.ExecutionConfiguration;
import org.fortiss.tooling.kernel.service.IExecutionService;
import org.fortiss.tooling.kernel.service.ITransformationService;
import org.fortiss.tooling.kernel.util.ExtensionPointUtils;
import org.fortiss.tooling.kernel.util.LoggingUtils;
import org.fortiss.tooling.kernel.util.ProjectRootElementUtils;
import org.osgi.framework.Bundle;

/**
 * This class implements the {@link IExecutionService} interface.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 7134EE4805A16FECA71F2023FEC81341
 */
public class ExecutionService implements IExecutionService {

	/** The execution target extension point ID. */
	private static final String EXTENSION_POINT_NAME = "org.fortiss.tooling.kernel.executionTarget";

	/** The execution target configuration element name. */
	private static final String CONFIGURATION_ELEMENT_NAME = "executionTarget";

	/** Stores the registered execution targets. */
	private final List<IExecutionTarget> executionTargetList = new ArrayList<IExecutionTarget>();

	/** Constructor. */
	public ExecutionService() {
		setupExecutionTargets();
	}

	/** {@inheritDoc} */
	@Override
	public UnmodifiableList<IExecutionTarget> getExecutionTargets() {
		return CollectionUtils.asUnmodifiable(executionTargetList);
	}

	/** {@inheritDoc} */
	@Override
	public UnmodifiableList<IExecutionTarget> getExecutionTargets(
			ExecutionConfiguration config) {
		List<IExecutionTarget> result = new LinkedList<IExecutionTarget>();
		for (IExecutionTarget target : executionTargetList) {
			Class<? extends Object> executableClass = target
					.getExecutableClass();
			EObject source = ProjectRootElementUtils.findElementById(config);
			if (source != null
					&& executableClass != null
					&& findTransformationChain(source, executableClass, config) != null) {
				result.add(target);
			}
		}
		return CollectionUtils.asUnmodifiable(result);
	}

	/** {@inheritDoc} */
	@Override
	public Object prepareExecutable(ExecutionConfiguration config,
			IExecutionTarget target) throws ChainTransformationFailedException {
		EObject source = ProjectRootElementUtils.findElementById(config);
		if (source != null) {
			List<TransformationProviderChain> chains = findTransformationChain(
					source, target.getExecutableClass(), config);
			return chains.get(0).transform(source,
					new ExecutionConfigTransformationContext(config));
		}
		return null;
	}

	/** Initializes the targets from plugin extensions. */
	private void setupExecutionTargets() {
		for (IConfigurationElement ce : ExtensionPointUtils
				.getConfigurationElements(EXTENSION_POINT_NAME,
						CONFIGURATION_ELEMENT_NAME)) {
			Bundle bundle = ExtensionPointUtils.getBundle(ce);
			try {
				Class<?> handlerClass = ExtensionPointUtils.loadClass(
						ce.getAttribute("target"), bundle);
				IExecutionTarget target = (IExecutionTarget) handlerClass
						.getConstructor().newInstance();
				executionTargetList.add(target);
			} catch (Exception ex) {
				LoggingUtils.error(ToolingKernelActivator.getDefault(),
						ex.getMessage(), ex);
			}
		}
	}

	/** Uses transformation service to find suitable chains of transformations. */
	List<TransformationProviderChain> findTransformationChain(EObject source,
			Class<? extends Object> executableClass,
			ExecutionConfiguration config) {
		return ITransformationService.INSTANCE.getTransformationProviderChain(
				source.getClass(), executableClass,
				new ExecutionConfigTransformationContext(config));
	}
}
