/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.service;

import org.conqat.lib.commons.collections.UnmodifiableList;
import org.fortiss.tooling.kernel.extension.IExecutionTarget;
import org.fortiss.tooling.kernel.extension.exception.ChainTransformationFailedException;
import org.fortiss.tooling.kernel.internal.ExecutionService;
import org.fortiss.tooling.kernel.model.ExecutionConfiguration;
import org.fortiss.tooling.kernel.model.ExecutionConfigurationPart;
import org.fortiss.tooling.kernel.model.IIdLabeled;

/**
 * This service provides execution configurations and target execution for
 * specific model elements and execution targets. Usually the execution service
 * makes use of the {@link ITransformationService} to transform an input model
 * element into an executable object depending on which execution target was
 * provided.
 * <P>
 * A typical example for an execution target is a simulator for a given modeling
 * language or a specific target language like Java for a code generator.
 * <P>
 * An execution configuration in its simplest form references an
 * {@link IIdLabeled} model element. However, an execution configuration can
 * also contain additional information for the execution, e.g, sub-classes of
 * {@link ExecutionConfigurationPart}. A typical example would be a set of
 * parameters provided to some transformation, which takes part in the creation
 * of the executable object (e.g a test case generator may be parameterized to
 * use different methods of test generation).
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 94DD29305B0923639A62E499C86520F0
 */
public interface IExecutionService {

	/** Returns the singleton instance of the execution service. */
	public static final IExecutionService INSTANCE = new ExecutionService();

	/** Returns an unmodifiable list of registered execution targets. */
	UnmodifiableList<IExecutionTarget> getExecutionTargets();

	/**
	 * Returns an unmodifiable list of possible execution targets using the
	 * given execution configuration.
	 */
	UnmodifiableList<IExecutionTarget> getExecutionTargets(
			ExecutionConfiguration config);

	/**
	 * Prepares an executable object using the given
	 * {@link ExecutionConfiguration} and the transformation service.
	 * 
	 * @throws ChainTransformationFailedException
	 *             if some part of the transformation chain fails.
	 */
	Object prepareExecutable(ExecutionConfiguration config,
			IExecutionTarget target) throws ChainTransformationFailedException;
}
